@@ -35,6 +35,11 @@ import os, platform, sys, re
 import pwd
 import gtk, pango
 
+try:
+  import gconf
+except:
+  pass
+
 # set this to true to enable debugging output
 # These should be moved somewhere better.
 debug = False
@@ -65,7 +70,7 @@ Defaults = {
   'delete_binding'        : 'delete-sequence',
   'cursor_blink'          : True,
   'emulation'             : 'xterm',
-  'font'                  : 'Mono',
+  'font'                  : 'Mono 10',
   'foreground_color'      : '#AAAAAA',
   'scrollbar_position'    : "right",
   'scroll_background'     : True,
@@ -109,6 +114,10 @@ Defaults = {
     'new_tab'          : '<Ctrl><Shift>T',
     'go_next'          : ('<Ctrl><Shift>N','<Ctrl>Tab'),
     'go_prev'          : ('<Ctrl><Shift>P','<Ctrl><Shift>Tab'),
+    'go_up'            : '<Alt>Up',
+    'go_down'          : '<Alt>Down',
+    'go_left'          : '<Alt>Left',
+    'go_right'         : '<Alt>Right',
     'split_horiz'      : '<Ctrl><Shift>O',
     'split_vert'       : '<Ctrl><Shift>E',
     'close_term'       : '<Ctrl><Shift>W',
@@ -128,6 +137,8 @@ Defaults = {
     'next_tab'         : '<Ctrl>Page_Down',
     'prev_tab'         : '<Ctrl>Page_Up',
     'full_screen'      : 'F11',
+    'reset'            : '<Ctrl><Shift>R',
+    'reset_clear'      : '<Ctrl><Shift>G',
   }
 }
 
@@ -183,8 +194,9 @@ class TerminatorConfValuestore:
   # Our settings
   def __getitem__ (self, keyname):
     if self.values.has_key (keyname):
-      dbg ("Returning '%s'"%keyname)
-      return self.values[keyname]
+      value = self.values[keyname]
+      dbg ("Returning '%s':'%s'"%(keyname, value))
+      return value
     else:
       dbg ("Failed to find '%s'"%keyname)
       raise (KeyError)
@@ -197,8 +209,6 @@ class TerminatorConfValuestoreDefault (TerminatorConfValuestore):
 class TerminatorConfValuestoreRC (TerminatorConfValuestore):
   rcfilename = ""
   type = "RCFile"
-  #FIXME: use inotify to watch the rc, split __init__ into a parsing function
-  #       that can be re-used when rc changes.
   def __init__ (self):
     try:
       directory = os.environ['XDG_CONFIG_HOME']
@@ -207,12 +217,25 @@ class TerminatorConfValuestoreRC (TerminatorConfValuestore):
       directory = os.path.join (os.path.expanduser("~"), ".config")
     self.rcfilename = os.path.join(directory, "terminator/config")
     dbg(" VS_RCFile: config file located at %s" % self.rcfilename)
-    if os.path.exists (self.rcfilename):
+    self.call_parser(True)
+
+  def set_reconfigure_callback (self, function):
+    dbg (" VS_RCFile: setting callback to: %s"%function)
+    self.reconfigure_callback = function
+    return (True)
+
+  def call_parser (self, is_init = False):
+    dbg (" VS_RCFile: parsing config file")
+    try:
       ini = ConfigFile(self.rcfilename, self._rc_set_callback())
-      try:
-        ini.parse()
-      except ParsedWithErrors, e:
-        msg = _("""<big><b>Configuration error</b></big>
+      ini.parse()
+    except IOError, e:
+      dbg (" VS_RCFile: unable to open %s (%r)" % (self.rcfilename, e))
+    except ParsedWithErrors, e:
+      # We don't really want to produce an error dialog every run
+      if not is_init:
+        pass
+      msg = _("""<big><b>Configuration error</b></big>
 
 Errors were encountered while parsing terminator_config(5) file:
 
@@ -220,56 +243,60 @@ Errors were encountered while parsing terminator_config(5) file:
 
 %d line(s) have been ignored.""") % (self.rcfilename, len(e.errors))
 
-        dialog = gtk.Dialog(_("Configuration error"), None, gtk.DIALOG_MODAL,
-                            (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
-        dialog.set_has_separator(False)
-        dialog.set_resizable(False)
+      dialog = gtk.Dialog(_("Configuration error"), None, gtk.DIALOG_MODAL,
+                          (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+      dialog.set_has_separator(False)
+      dialog.set_resizable(False)
 
-        image = gtk.image_new_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_DIALOG)
-        image.set_alignment (0.5, 0)
-        dmsg = gtk.Label(msg)
-        dmsg.set_use_markup(True)
-        dmsg.set_alignment(0, 0.5)
+      image = gtk.image_new_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_DIALOG)
+      image.set_alignment (0.5, 0)
+      dmsg = gtk.Label(msg)
+      dmsg.set_use_markup(True)
+      dmsg.set_alignment(0, 0.5)
 
-        textbuff = gtk.TextBuffer()
-        textbuff.set_text("\n".join(map(lambda e: str(e), e.errors)))
-        textview = gtk.TextView(textbuff)
-        textview.set_editable(False)
+      textbuff = gtk.TextBuffer()
+      textbuff.set_text("\n".join(map(lambda e: str(e), e.errors)))
+      textview = gtk.TextView(textbuff)
+      textview.set_editable(False)
 
-        textview.modify_font(pango.FontDescription(Defaults['font']))
-        textscroll = gtk.ScrolledWindow()
-        textscroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        textscroll.add(textview)
-        # This should be scaled with the size of the text and font
-        textscroll.set_size_request(600, 200)
+      textview.modify_font(pango.FontDescription(Defaults['font']))
+      textscroll = gtk.ScrolledWindow()
+      textscroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+      textscroll.add(textview)
+      # This should be scaled with the size of the text and font
+      textscroll.set_size_request(600, 200)
 
-        root = gtk.VBox()
-        root.pack_start(dmsg, padding = 6)
-        root.pack_start(textscroll, padding = 6)
+      root = gtk.VBox()
+      root.pack_start(dmsg, padding = 6)
+      root.pack_start(textscroll, padding = 6)
 
-        box = gtk.HBox()
-        box.pack_start (image, False, False, 6)
-        box.pack_start (root, False, False, 6)
-        dialog.vbox.pack_start (box, False, False, 12)
-        dialog.show_all()
+      box = gtk.HBox()
+      box.pack_start (image, False, False, 6)
+      box.pack_start (root, False, False, 6)
+      dialog.vbox.pack_start (box, False, False, 12)
+      dialog.show_all()
 
-        dialog.run()
-        dialog.destroy()
+      dialog.run()
+      dialog.destroy()
 
-    dbg("ConfigFile settings are: %s" % repr(self.values))
+    dbg("ConfigFile settings are: %r" % self.values)
 
   def _rc_set_callback(self):
-    def callback(section, key, value):
+    def callback(sections, key, value):
+      dbg("Setting: section=%r with %r => %r" % (sections, key, value))
+      section = None
+      if len(sections) > 0:
+        section = sections[0]
       if section is None:
         if not Defaults.has_key (key):
-          raise ValueError("Unknown configuration option %s" % repr(key))
+          raise ValueError("Unknown configuration option %r" % key)
         deftype = Defaults[key].__class__.__name__
         if key.endswith('_color'):
           try:
             gtk.gdk.color_parse(value)
             self.values[key] = value
           except ValueError:
-            raise ValueError(_("Setting %s value %s not a valid colour; ignoring") % (key,repr(value)))
+            raise ValueError(_("Setting %r value %r not a valid colour; ignoring") % (key, value))
         elif key == 'tab_position':
           if value.lower() in ('top', 'left', 'bottom', 'right'):
             self.values[key] = value.lower()
@@ -290,28 +317,29 @@ Errors were encountered while parsing terminator_config(5) file:
           raise ValueError(_("Reading list values from terminator_config(5) is not currently supported"))
         elif deftype == 'dict':
           if type(value) != dict:
-            raise ValueError(_("Setting %s should be a section name") % repr(key))
+            raise ValueError(_("Setting %r should be a section name") % key)
           self.values[key] = value
         else:
           self.values[key] = value
 
-        dbg (" VS_RCFile: Set value '%s' to %s" % (key, repr(self.values[key])))
+        dbg (" VS_RCFile: Set value %r to %r" % (key, self.values[key]))
       elif section == 'keybindings':
         self.values.setdefault(section, {})
         if not Defaults[section].has_key(key):
-          raise ValueError("Keybinding name %s is unknown" % repr(key))
+          raise ValueError("Keybinding name %r is unknown" % key)
         else:
           self.values[section][key] = value
       else:
-        raise ValueError("Section name %s is unknown" % repr(section))
+        raise ValueError("Section name %r is unknown" % section)
     return callback
 
 class TerminatorConfValuestoreGConf (TerminatorConfValuestore):
   profile = ""
   client = None
   cache = {}
+  notifies = {}
 
-  def __init__ (self, profile = None):
+  def __init__ (self, profileName = None):
     self.type = "GConf"
     self.inactive = False
 
@@ -323,16 +351,23 @@ class TerminatorConfValuestoreGConf (TerminatorConfValuestore):
     self._gt_dir = Defaults['gt_dir']
     self._profile_dir = Defaults['profile_dir']
 
-    dbg ('VSGConf: Profile requested is: "%s"'%profile)
-    if not profile:
-      profile = self.client.get_string (self._gt_dir + '/global/default_profile')
-    dbg ('VSGConf: Profile bet on is: "%s"'%profile)
+    dbg ('VSGConf: Profile bet on is: "%s"'%profileName)
     profiles = self.client.get_list (self._gt_dir + '/global/profile_list','string')
     dbg ('VSGConf: Found profiles: "%s"'%profiles)
 
-    #set up the active encoding list
-    self.active_encodings = self.client.get_list (self._gt_dir + '/global/active_encodings', 'string')
-    
+    dbg ('VSGConf: Profile requested is: "%s"'%profileName)
+    if not profileName:
+      profile = self.client.get_string (self._gt_dir + '/global/default_profile')
+    else:
+      profile = profileName
+      # In newer gnome-terminal, the profile keys are named Profile0/1 etc.
+      # We have to match using visible_name instead
+      for p in profiles:
+        profileName2 = self.client.get_string (
+          self._profile_dir + '/' + p + '/visible_name')
+        if profileName == profileName2:
+          profile = p
+
     #need to handle the list of Gconf.value
     if profile in profiles:
       dbg (" VSGConf: Found profile '%s' in profile_list"%profile)
@@ -347,6 +382,9 @@ class TerminatorConfValuestoreGConf (TerminatorConfValuestore):
       self.inactive = True
       return
 
+    #set up the active encoding list
+    self.active_encodings = self.client.get_list (self._gt_dir + '/global/active_encodings', 'string')
+    
     self.client.add_dir (self.profile, gconf.CLIENT_PRELOAD_RECURSIVE)
     if self.on_gconf_notify:
       self.client.notify_add (self.profile, self.on_gconf_notify)
@@ -365,8 +403,12 @@ class TerminatorConfValuestoreGConf (TerminatorConfValuestore):
   def on_gconf_notify (self, client, cnxn_id, entry, what):
     dbg (" VSGConf: invalidating cache")
     self.cache = {}
-    dbg (" VSGConf: gconf changed, callback is: %s"%self.reconfigure_callback)
+    dbg (" VSGConf: gconf changed, may run a callback. %s, %s"%(entry.key, entry.value))
+    if entry.key[-12:] == 'visible_name':
+      dbg (" VSGConf: only a visible_name change, ignoring")
+      return False
     if self.reconfigure_callback:
+      dbg (" VSGConf: callback is: %s"%self.reconfigure_callback)
       self.reconfigure_callback ()
 
   def __getitem__ (self, key = ""):
@@ -403,12 +445,26 @@ class TerminatorConfValuestoreGConf (TerminatorConfValuestore):
           value = 'http://%s:%s/'%(
             self.client.get_string ('/system/http_proxy/host'),
             self.client.get_int ('/system/http_proxy/port'))
+    elif key == 'cursor_blink':
+      tmp = self.client.get_string('%s/cursor_blink_mode' % self.profile)
+      if tmp in ['on', 'off'] and self.notifies.has_key ('cursor_blink'):
+        self.client.notify_remove (self.notifies['cursor_blink'])
+        del (self.notifies['cursor_blink'])
+      if tmp == 'on':
+        value = True
+      elif tmp == 'off':
+        value = False
+      elif tmp == 'system':
+        value = self.client.get_bool ('/desktop/gnome/interface/cursor_blink')
+        self.notifies['cursor_blink'] = self.client.notify_add ('/desktop/gnome/interface/cursor_blink', self.on_gconf_notify)
+      else:
+        value = self.client.get ('%s/%s'%(self.profile, key))
     else:
       value = self.client.get ('%s/%s'%(self.profile, key))
 
-    if value:
-      from types import StringType
-      if type(value) is StringType:
+    if value != None:
+      from types import StringType,BooleanType
+      if type(value) in [StringType, BooleanType]:
          ret = value
       else:
         funcname = "get_" + Defaults[key].__class__.__name__
@@ -427,36 +483,3 @@ class TerminatorConfValuestoreGConf (TerminatorConfValuestore):
     else:
       raise (KeyError)
 
-if __name__ == '__main__':
-
-  stores = []
-  stores.append (TerminatorConfValuestoreRC ())
-
-  try:
-    import gconf
-    stores.append (TerminatorConfValuestoreGConf ())
-  except:
-    pass
-
-  foo = TerminatorConfig (stores)
-
-  ## cmsj: this is my testing ground
-  ##       ensure that font is set in the Default gconf profile
-  ##       set titlebars in the RC file
-  ##       remove titletips from gconf/RC
-  ##       do not define blimnle in any way
-
-  # This should come from gconf (it's set by gnome-terminal)
-  print foo.font
-
-  # This should come from RC
-  print foo.titlebars
-
-  # This should come from defaults
-  print foo.titletips
-
-  # This should raise AttributeError
-  #print foo.blimnle
-
-  # http_proxy is a value that is allowed to not exist
-  print "final proxy: %s"%foo.http_proxy
