@@ -29,35 +29,61 @@ from config import dbg, err, debug
 
 from terminatorlib.keybindings import TerminatorKeybindings
 from terminatorlib.terminatorterm import TerminatorTerm
+from terminatorlib.prefs_profile import ProfileEditor
 
 class TerminatorNotebookTabLabel(gtk.HBox):
+  _terminator = None
+  _notebook = None
+  _label = None
+  _icon = None
+  _button = None
+    
   def __init__(self, title, notebook, terminator):
     gtk.HBox.__init__(self, False)
     self._notebook = notebook
-    self.terminator = terminator
+    self._terminator = terminator
+    
     self._label = gtk.Label(title)
-    tab_pos = notebook.get_tab_pos()
-    if tab_pos == gtk.POS_LEFT:
-      self._label.set_angle(90)
-    elif tab_pos == gtk.POS_RIGHT:
-      self._label.set_angle(270)
-    icon = gtk.Image()
-    icon.set_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
+    self.update_angle()
     self.pack_start(self._label, True, True)
 
-    if terminator.conf.close_button_on_tab:
-      self._button = gtk.Button()
+    self._icon = gtk.Image()
+    self._icon.set_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
+    
+    self.update_closebut()
+
+    self.show_all()
+
+  def update_closebut(self):
+    if self._terminator.conf.close_button_on_tab:
+      if not self._button:
+        self._button = gtk.Button()
       self._button.set_relief(gtk.RELIEF_NONE)
       self._button.set_focus_on_click(False)
       self._button.set_relief(gtk.RELIEF_NONE)
-      self._button.add(icon)
+      self._button.add(self._icon)
       self._button.connect('clicked', self.on_close)
       self._button.set_name("terminator-tab-close-button")
       self.connect("style-set", self.on_style_set)
       if hasattr(self._button, "set_tooltip_text"): 
         self._button.set_tooltip_text(_("Close Tab"))
-      self.pack_start(self._button, False, False)
-    self.show_all()
+        self.pack_start(self._button, False, False)
+        self.show_all()
+    else:
+      if self._button:
+        self._button.remove(self._icon)
+        self.remove(self._button)
+        del(self._button)
+        self._button = None
+
+  def update_angle(self):
+      tab_pos = self._notebook.get_tab_pos()
+      if tab_pos == gtk.POS_LEFT:
+        self._label.set_angle(90)
+      elif tab_pos == gtk.POS_RIGHT:
+        self._label.set_angle(270)
+      else:
+        self._label.set_angle(0)
 
   def on_style_set(self, widget, prevstyle):
     x, y = gtk.icon_size_lookup_for_settings( self._button.get_settings(), gtk.ICON_SIZE_MENU)
@@ -68,13 +94,16 @@ class TerminatorNotebookTabLabel(gtk.HBox):
     for i in xrange(0,nbpages):
       if self._notebook.get_tab_label(self._notebook.get_nth_page(i)) == self:
         #dbg("[Close from tab] Found tab at position [%d]" % i)
-        term = self.terminator._notebook_first_term(self._notebook.get_nth_page(i))
+        if not isinstance (self._notebook.get_nth_page(i), TerminatorTerm):
+          if self._terminator.confirm_close_multiple (self._terminator.window, _("tab")):
+            return False
+        term = self._terminator._notebook_first_term(self._notebook.get_nth_page(i))
         while term:
           if term == self._notebook.get_nth_page(i):
-            self.terminator.closeterm(term)
+            self._terminator.closeterm(term)
             break
-          self.terminator.closeterm(term)
-          term = self.terminator._notebook_first_term(self._notebook.get_nth_page(i))
+          self._terminator.closeterm(term)
+          term = self._terminator._notebook_first_term(self._notebook.get_nth_page(i))
         break
 
   def set_title(self, title):
@@ -89,8 +118,10 @@ class TerminatorNotebookTabLabel(gtk.HBox):
   def width_request(self):
     return self.size_request()[0]
 
-
 class Terminator:
+  options = None
+  groupings = []
+
   def __init__ (self, profile = None, command = None, fullscreen = False,
                 maximise = False, borderless = False, no_gconf = False,
                 geometry = None):
@@ -104,8 +135,12 @@ class Terminator:
     self.debugaddress = None
     self.start_cwd = os.getcwd()
     self.term_list = []
+    self.gnome_client = None
     stores = []
-    stores.append (config.TerminatorConfValuestoreRC ())
+
+    store = config.TerminatorConfValuestoreRC ()
+    store.set_reconfigure_callback (self.reconfigure_vtes)
+    stores.append (store)
     
     self._tab_reorderable = True
     if not hasattr(gtk.Notebook, "set_tab_reorderable") or not hasattr(gtk.Notebook, "get_tab_reorderable"):
@@ -177,24 +212,7 @@ class Terminator:
       print "full_screen = <Ctrl><Shift>F11"
     self.keybindings.configure(self.conf.keybindings)
 
-    if self.conf.handle_size in xrange (0,6):
-      gtk.rc_parse_string("""
-        style "terminator-paned-style" {
-            GtkPaned::handle_size = %s 
-        }
-        
-        class "GtkPaned" style "terminator-paned-style"
-        """ % self.conf.handle_size)
-        
-    gtk.rc_parse_string("""
-      style "terminator-tab-close-button-style" {
-            GtkWidget::focus-padding = 0
-            GtkWidget::focus-line-width = 0
-            xthickness = 0
-            ythickness = 0
-         }
-         widget "*.terminator-tab-close-button" style "terminator-tab-close-button-style"
-         """)
+    self.set_handle_size (self.conf.handle_size)
 
     self.window = gtk.Window ()
     self.window.set_title (APP_NAME.capitalize())
@@ -202,7 +220,7 @@ class Terminator:
     if self._geometry is not None:
       dbg("Geometry=%s" % self._geometry)
       if not self.window.parse_geometry(self._geometry):
-        err(_("Invalid geometry string %s") % repr(self._geometry))
+        err(_("Invalid geometry string %r") % self._geometry)
 
     try:
       self.window.set_icon (self.icon_theme.load_icon (APP_NAME, 48, 0))
@@ -229,10 +247,8 @@ class Terminator:
     # Set RGBA colormap if possible so VTE can use real alpha
     # channels for transparency.
     if self.conf.enable_real_transparency:
-      screen = self.window.get_screen()
-      colormap = screen.get_rgba_colormap()
-      if colormap:
-        self.window.set_colormap(colormap)
+      dbg ('H9TRANS: Enabling real transparency')
+      self.enable_rgba(True)
 
     # Start out with just one terminal
     # FIXME: This should be really be decided from some kind of profile
@@ -245,6 +261,35 @@ class Terminator:
     term.spawn_child ()
     self.save_yourself ()
 
+  def set_handle_size (self, size):
+    if size in xrange (0,6):
+      gtk.rc_parse_string("""
+        style "terminator-paned-style" {
+            GtkPaned::handle_size = %s 
+        }
+        
+        class "GtkPaned" style "terminator-paned-style"
+        """ % self.conf.handle_size)
+        
+      gtk.rc_parse_string("""
+      style "terminator-tab-close-button-style" {
+            GtkWidget::focus-padding = 0
+            GtkWidget::focus-line-width = 0
+            xthickness = 0
+            ythickness = 0
+         }
+         widget "*.terminator-tab-close-button" style "terminator-tab-close-button-style"
+         """)
+
+  def enable_rgba (self, rgba = False):
+    screen = self.window.get_screen()
+    if rgba:
+      colormap = screen.get_rgba_colormap()
+    else:
+      colormap = screen.get_rgb_colormap()
+    if colormap:
+      self.window.set_colormap(colormap)
+
   def die(self, *args):
     gtk.main_quit ()
 
@@ -272,31 +317,36 @@ class Terminator:
     if self._fullscreen:
       args.append('--fullscreen')
 
-    # We can't set an interpreter because Gnome unconditionally spams it with
-    # --sm-foo-bar arguments before our own argument list. *mutter*
-    # So, hopefully your #! line is correct.  If not, we could write out
-    # a shell script with the interpreter name etc.
-    c = self.gnome_client
-    c.set_program(sys.argv[0])
-    dbg("Session restart command: %s with args %s in %s" % (sys.argv[0], repr(args), self.start_cwd))
+    if self.gnome_client:
+      # We can't set an interpreter because Gnome unconditionally spams it with
+      # --sm-foo-bar arguments before our own argument list. *mutter*
+      # So, hopefully your #! line is correct.  If not, we could write out
+      # a shell script with the interpreter name etc.
+      c = self.gnome_client
+      c.set_program(sys.argv[0])
+      dbg("Session restart command: %s with args %r in %s" % (sys.argv[0], args, self.start_cwd))
 
-    c.set_restart_style(gnome.ui.RESTART_IF_RUNNING)
-    c.set_current_directory(self.start_cwd)
-    try:
-      c.set_restart_command(args)
-      c.set_clone_command(args)
-    except (TypeError,AttributeError):
-      # Apparantly needed for some Fedora systems
-      # see http://trac.nicfit.net/mesk/ticket/137
-      dbg("Gnome bindings have weird set_clone/restart_command")
-      c.set_restart_command(len(args), args)
-      c.set_clone_command(len(args), args)
-    return True
+      c.set_restart_style(gnome.ui.RESTART_IF_RUNNING)
+      c.set_current_directory(self.start_cwd)
+      try:
+        c.set_restart_command(args)
+        c.set_clone_command(args)
+      except (TypeError,AttributeError):
+        # Apparantly needed for some Fedora systems
+        # see http://trac.nicfit.net/mesk/ticket/137
+        dbg("Gnome bindings have weird set_clone/restart_command")
+        c.set_restart_command(len(args), args)
+        c.set_clone_command(len(args), args)
+      return True
 
   def maximize (self):
     """ Maximize the Terminator window."""
     self.window.maximize ()
 
+  def unmaximize (self):
+    """ Unmaximize the Terminator window."""
+    self.window.unmaximize ()
+
   def fullscreen_toggle (self):
     """ Toggle the fullscreen state of the window. If it is in
         fullscreen state, it will be unfullscreened. If it is not, it
@@ -307,6 +357,12 @@ class Terminator:
     else:
       self.window.fullscreen ()
 
+  def fullscreen_absolute (self, fullscreen):
+    """ Explicitly set the fullscreen state of the window.
+    """
+    if self._fullscreen != fullscreen:
+      self.fullscreen_toggle ()
+
   def on_window_state_changed (self, window, event):
     self._fullscreen = bool (event.new_window_state & gtk.gdk.WINDOW_STATE_FULLSCREEN)
     self._maximised = bool (event.new_window_state & gtk.gdk.WINDOW_STATE_MAXIMIZED)
@@ -316,7 +372,9 @@ class Terminator:
   def on_delete_event (self, window, event, data=None):
     if len (self.term_list) == 1:
       return False
+    return self.confirm_close_multiple (window, _("window"))
 
+  def confirm_close_multiple (self, window, type):
     # show dialog
     dialog = gtk.Dialog (_("Close?"), window, gtk.DIALOG_MODAL)
     dialog.set_has_separator (False)
@@ -324,12 +382,12 @@ class Terminator:
 
     cancel = dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT)
     close_all = dialog.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_ACCEPT)
-    label = close_all.get_children()[0].get_children()[0].get_children()[1].set_label(_("Close All _Terminals"))
+    label = close_all.get_children()[0].get_children()[0].get_children()[1].set_label(_("Close _Terminals"))
 
-    primairy = gtk.Label (_('<big><b>Close all terminals?</b></big>'))
+    primairy = gtk.Label (_('<big><b>Close multiple terminals?</b></big>'))
     primairy.set_use_markup (True)
     primairy.set_alignment (0, 0.5)
-    secundairy = gtk.Label (_("This window has %s terminals open.  Closing the window will also close all terminals.") % len(self.term_list))
+    secundairy = gtk.Label (_("This %s has several terminals open.  Closing the %s will also close all terminals within it.") % (type, type))
     secundairy.set_line_wrap(True)
     primairy.set_alignment (0, 0.5)
 
@@ -364,13 +422,15 @@ class Terminator:
     mapping = self.keybindings.lookup(event)
 
     if mapping:
-      dbg("on_key_press: lookup found %s" % repr(mapping))
+      dbg("on_key_press: lookup found %r" % mapping)
       if mapping == 'full_screen':
         self.fullscreen_toggle ()
-        return (True)
       elif mapping == 'close_window':
         if not self.on_delete_event (window, gtk.gdk.Event (gtk.gdk.DELETE)):
           self.on_destroy_event (window, gtk.gdk.Event (gtk.gdk.DESTROY))
+      else:
+        return (False)
+      return (True)
 
   def set_window_title(self, title):
     """
@@ -664,9 +724,9 @@ class Terminator:
     dbg ('SEGBUG: showed TerminatorTerm')
     terminal.spawn_child ()
     dbg ('SEGBUG: spawned child')
-    return terminal
+    return
   
-  def remove(self, widget):
+  def remove(self, widget, keep = False):
     """Remove a TerminatorTerm from the Terminator view and terms list
        Returns True on success, False on failure"""
     parent = widget.get_parent ()
@@ -676,7 +736,7 @@ class Terminator:
       # We are the only term
       if not self.on_delete_event (parent, gtk.gdk.Event (gtk.gdk.DELETE)):
         self.on_destroy_event (parent, gtk.gdk.Event (gtk.gdk.DESTROY))
-      return
+      return True
 
     if isinstance (parent, gtk.Paned):
       index = self.term_list.index (widget)
@@ -714,17 +774,23 @@ class Terminator:
         sibling.reparent (grandparent)
         if not self._zoomed:
           grandparent.resize_children()
-      parent.destroy ()
       if isinstance(sibling, TerminatorTerm) and isinstance(sibling.get_parent(), gtk.Notebook):
         sibling._titlebox.hide()
-        
+
       self.term_list.remove (widget)
+      if not keep:
+        widget._vte.get_parent().remove(widget._vte)
+        widget._vte = None
 
     elif isinstance (parent, gtk.Notebook):
       parent.remove(widget)
       nbpages = parent.get_n_pages()
       index = self.term_list.index (widget)
+
       self.term_list.remove (widget)
+      if not keep:
+        widget._vte.get_parent().remove(widget._vte)
+        widget._vte = None
       if nbpages == 1:
         if self.window.allocation.height != gtk.gdk.screen_height():
           self.window.resize(self.window.allocation.width, min(self.window.allocation.height - parent.get_tab_label(parent.get_nth_page(0)).height_request(), gtk.gdk.screen_height()))
@@ -743,7 +809,6 @@ class Terminator:
             gdparent.pack2(sibling)
         if isinstance(sibling, TerminatorTerm) and sibling.conf.titlebars and sibling.conf.extreme_tabs:
           sibling._titlebox.show()
-        parent.destroy()
     if self.conf.focus_on_close == 'prev' or ( self.conf.focus_on_close == 'auto' and focus_on_close == 'prev'):
       if index == 0: index = 1
       self.term_list[index - 1]._vte.grab_focus ()
@@ -765,11 +830,205 @@ class Terminator:
       self.toggle_zoom(widget, not self._maximised)
 
     if self.remove(widget):
-      widget.destroy ()
+      self.group_hoover()
       return True
     return False
 
+  def go_to (self, term, selector):
+    current = self.term_list.index (term)
+    target = selector (term)
+    if not target is None:
+        term = self.term_list[target]
+        ##we need to set the current page of each notebook
+        self._set_current_notebook_page_recursive(term)
+        term._vte.grab_focus ()
+
+  def _select_direction (self, term, matcher):
+    current = self.term_list.index (term)
+    current_geo = term.get_geometry ()
+    best_index = None
+    best_geo = None
+
+    for i in range(0,len(self.term_list)):
+        if i == current:
+            continue
+        possible = self.term_list[i]
+        possible_geo = possible.get_geometry ()
+
+        #import pprint
+        #print "I am %d" % (current)
+        #pprint.pprint(current_geo)
+        #print "I saw %d" % (i)
+        #pprint.pprint(possible_geo)
+
+        if matcher (current_geo, possible_geo, best_geo):
+            best_index = i
+            best_geo = possible_geo
+    #if best_index is None:
+    #    print "nothing best"
+    #else:
+    #    print "sending %d" % (best_index)
+    return best_index
+
+  def _match_up (self, current_geo, possible_geo, best_geo):
+    '''We want to find terminals that are fully above the top
+       border, but closest in the y direction, breaking ties via
+       the closest cursor x position.'''
+    #print "matching up..."
+    # top edge of the current terminal
+    edge = current_geo['origin_y']
+    # botoom edge of the possible target
+    new_edge = possible_geo['origin_y']+possible_geo['span_y']
+    if new_edge < edge:
+        #print "new_edge < edge"
+        if best_geo is None:
+            #print "first thing left"
+            return True
+        best_edge = best_geo['origin_y']+best_geo['span_y']
+        if new_edge > best_edge:
+            #print "closer y"
+            return True
+        if new_edge == best_edge:
+            #print "same y"
+
+            cursor      = current_geo['origin_x']  + current_geo['cursor_x']
+            new_cursor  = possible_geo['origin_x'] + possible_geo['cursor_x']
+            best_cursor = best_geo['origin_x']     + best_geo['cursor_x']
+
+            if abs(new_cursor - cursor) < abs(best_cursor - cursor):
+                #print "closer x"
+                return True
+    #print "fail"
+    return False
+
+  def _match_down (self, current_geo, possible_geo, best_geo):
+    '''We want to find terminals that are fully below the bottom
+       border, but closest in the y direction, breaking ties via
+       the closest cursor x position.'''
+    #print "matching down..."
+    # bottom edge of the current terminal
+    edge = current_geo['origin_y']+current_geo['span_y']
+    # top edge of the possible target
+    new_edge = possible_geo['origin_y']
+    #print "edge: %d new_edge: %d" % (edge, new_edge)
+    if new_edge > edge:
+        #print "new_edge > edge"
+        if best_geo is None:
+            #print "first thing right"
+            return True
+        best_edge = best_geo['origin_y']
+        #print "best_edge: %d" % (best_edge)
+        if new_edge < best_edge:
+            #print "closer y"
+            return True
+        if new_edge == best_edge:
+            #print "same y"
+
+            cursor      = current_geo['origin_x']  + current_geo['cursor_x']
+            new_cursor  = possible_geo['origin_x'] + possible_geo['cursor_x']
+            best_cursor = best_geo['origin_x']     + best_geo['cursor_x']
+
+            if abs(new_cursor - cursor) < abs(best_cursor - cursor):
+                #print "closer x"
+                return True
+    #print "fail"
+    return False
+
+  def _match_left (self, current_geo, possible_geo, best_geo):
+    '''We want to find terminals that are fully to the left of
+       the left-side border, but closest in the x direction, breaking
+       ties via the closest cursor y position.'''
+    #print "matching left..."
+    # left-side edge of the current terminal
+    edge = current_geo['origin_x']
+    # right-side edge of the possible target
+    new_edge = possible_geo['origin_x']+possible_geo['span_x']
+    if new_edge < edge:
+        #print "new_edge(%d) < edge(%d)" % (new_edge, edge)
+        if best_geo is None:
+            #print "first thing left"
+            return True
+        best_edge = best_geo['origin_x']+best_geo['span_x']
+        if new_edge > best_edge:
+            #print "closer x (new_edge(%d) > best_edge(%d))" % (new_edge, best_edge)
+            return True
+        if new_edge == best_edge:
+            #print "same x"
+
+            cursor      = current_geo['origin_y']  + current_geo['cursor_y']
+            new_cursor  = possible_geo['origin_y'] + possible_geo['cursor_y']
+            best_cursor = best_geo['origin_y']     + best_geo['cursor_y']
+
+            if abs(new_cursor - cursor) < abs(best_cursor - cursor):
+                #print "closer y"
+                return True
+    #print "fail"
+    return False
+
+  def _match_right (self, current_geo, possible_geo, best_geo):
+    '''We want to find terminals that are fully to the right of
+       the right-side border, but closest in the x direction, breaking
+       ties via the closest cursor y position.'''
+    #print "matching right..."
+    # right-side edge of the current terminal
+    edge = current_geo['origin_x']+current_geo['span_x']
+    # left-side edge of the possible target
+    new_edge = possible_geo['origin_x']
+    #print "edge: %d new_edge: %d" % (edge, new_edge)
+    if new_edge > edge:
+        #print "new_edge > edge"
+        if best_geo is None:
+            #print "first thing right"
+            return True
+        best_edge = best_geo['origin_x']
+        #print "best_edge: %d" % (best_edge)
+        if new_edge < best_edge:
+            #print "closer x"
+            return True
+        if new_edge == best_edge:
+            #print "same x"
+
+            cursor      = current_geo['origin_y']  + current_geo['cursor_y']
+            new_cursor  = possible_geo['origin_y'] + possible_geo['cursor_y']
+            best_cursor = best_geo['origin_y']     + best_geo['cursor_y']
+
+            if abs(new_cursor - cursor) < abs(best_cursor - cursor):
+                #print "closer y"
+                return True
+    #print "fail"
+    return False
+
+  def _select_up (self, term):
+    return self._select_direction (term, self._match_up)
+
+  def _select_down (self, term):
+    return self._select_direction (term, self._match_down)
+
+  def _select_left (self, term):
+    return self._select_direction (term, self._match_left)
+
+  def _select_right (self, term):
+    return self._select_direction (term, self._match_right)
+
   def go_next (self, term):
+    self.go_to (term, self._select_next)
+
+  def go_prev (self, term):
+    self.go_to (term, self._select_prev)
+
+  def go_up (self, term):
+    self.go_to (term, self._select_up)
+
+  def go_down (self, term):
+    self.go_to (term, self._select_down)
+
+  def go_left (self, term):
+    self.go_to (term, self._select_left)
+
+  def go_right (self, term):
+    self.go_to (term, self._select_right)
+
+  def _select_next (self, term):
     current = self.term_list.index (term)
     next = None
     if self.conf.cycle_term_tab:
@@ -779,23 +1038,17 @@ class Terminator:
         first = self._notebook_first_term(notebookpage[1])
         if term == last:
           next = self.term_list.index(first)
-        
+
     if next is None:
       if current == len (self.term_list) - 1:
         next = 0
       else:
         next = current + 1
+    return next
 
-    nextterm = self.term_list[next]
-    ##we need to set the current page of each notebook
-    self._set_current_notebook_page_recursive(nextterm)
-    
-    nextterm._vte.grab_focus ()
-
-  def go_prev (self, term):
+  def _select_prev (self, term):
     current = self.term_list.index (term)
     previous = None
-   
     if self.conf.cycle_term_tab:
       notebookpage = self.get_first_notebook_page(term)
       if notebookpage:
@@ -809,12 +1062,7 @@ class Terminator:
         previous = len (self.term_list) - 1
       else:
         previous = current - 1
-
-    #self.window.set_title(self.term_list[previous]._vte.get_window_title())
-    previousterm = self.term_list[previous]
-    ##we need to set the current page of each notebook
-    self._set_current_notebook_page_recursive(previousterm)
-    previousterm._vte.grab_focus ()
+    return previous
 
   def _set_current_notebook_page_recursive(self, widget):
     page = self.get_first_notebook_page(widget)
@@ -856,14 +1104,29 @@ class Terminator:
 
   def previous_tab(self, term):
     notebook = self.get_first_parent_notebook(term)
-    notebook.prev_page()
-    return
-    
+    if notebook:
+      cur = notebook.get_current_page()
+      pages = notebook.get_n_pages()
+      if cur == 0:
+        notebook.set_current_page(pages - 1)
+      else:
+        notebook.prev_page()
+      # This seems to be required in some versions of (py)gtk.
+      # Without it, the selection changes, but the displayed page doesn't change
+      # Seen in gtk-2.12.11 and pygtk-2.12.1 at least.
+      notebook.set_current_page(notebook.get_current_page())
+
   def next_tab(self, term):
     notebook = self.get_first_parent_notebook(term)
-    notebook.next_page()
-    return
-  
+    if notebook:
+      cur = notebook.get_current_page()
+      pages = notebook.get_n_pages()
+      if cur == pages - 1:
+        notebook.set_current_page(0)
+      else:
+        notebook.next_page()
+      notebook.set_current_page(notebook.get_current_page())
+
   def move_tab(self, term, direction):
     dbg("moving to direction %s" % direction)
     (notebook, page) = self.get_first_notebook_page(term)
@@ -1028,3 +1291,26 @@ class Terminator:
 
       widget._vte.grab_focus ()
 
+  def edit_profile (self, widget):
+    if not self.options:
+      self.options = ProfileEditor(self)
+      self.options.go()
+ 
+  def group_emit (self, terminatorterm, group, type, event):
+    for term in self.term_list:
+      if term != terminatorterm and term._group == group:
+        term._vte.emit (type, event)
+
+  def group_hoover (self):
+    destroy = []
+    for group in self.groupings:
+      save = False
+      for term in self.term_list:
+        if term._group == group:
+          save = True
+
+      if not save:
+        destroy.append (group)
+
+    for group in destroy:
+      self.groupings.remove (group)
