@@ -19,7 +19,7 @@
 import pygtk
 pygtk.require ("2.0")
 import gobject, gtk, pango
-import os, sys, subprocess, pwd
+import os, signal, sys, subprocess, pwd, re
 
 #import version details
 from terminatorlib.version import *
@@ -45,12 +45,15 @@ class TerminatorTerm (gtk.VBox):
   matches = {}
   TARGET_TYPE_VTE = 8
   _custom_font_size = None
+  _group = None
+  _want_titlebar = False
 
   def __init__ (self, terminator, profile = None, command = None, cwd = None):
     gtk.VBox.__init__ (self)
     self.terminator = terminator
     self.conf = terminator.conf
     self.command = command
+    self._oldtitle = ""
 
     self.cwd = cwd or os.getcwd();
     if not os.path.exists(self.cwd) or not os.path.isdir(self.cwd):
@@ -73,8 +76,15 @@ class TerminatorTerm (gtk.VBox):
     self._termbox.show()
     self._title = gtk.Label()
     self._title.show()
-    self._titlebox =  gtk.EventBox()
-    self._titlebox.add(self._title)
+    self._titlegroup = gtk.Label()
+    self._titlesep = gtk.VSeparator ()
+    self._titlebox = gtk.EventBox ()
+    self._titlehbox = gtk.HBox()
+    self._titlehbox.pack_start (self._titlegroup, False, True)
+    self._titlehbox.pack_start (self._titlesep, False, True, 2)
+    self._titlehbox.pack_start (self._title, True, True)
+    self._titlehbox.show ()
+    self._titlebox.add (self._titlehbox)
 
     self._search_string = None
     self._searchbox = gtk.HBox()
@@ -86,14 +96,34 @@ class TerminatorTerm (gtk.VBox):
     self._searchinput.connect('key-press-event', self.search_keypress)
 
     slabel = gtk.Label()
-    slabel.set_text("Finding text:")
+    slabel.set_text(_("Search:"))
     slabel.show()
+
+    sclose = gtk.Button()
+    sclose.set_relief(gtk.RELIEF_NONE)
+    sclose.set_focus_on_click(False)
+    sclose.set_relief(gtk.RELIEF_NONE)
+    sclose_icon = gtk.Image()
+    sclose_icon.set_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
+    sclose.add(sclose_icon)
+    sclose.set_name("terminator-search-close-button")
+    if hasattr(sclose, "set_tooltip_text"):
+      sclose.set_tooltip_text("Close Search Bar")
+    sclose.connect('clicked', self.end_search)
+    sclose.show_all()
+
+    # Button for the next result. Explicitly not show()n by default.
+    self._search_next = gtk.Button(_("Next"))
+    self._search_next.connect('clicked', self.next_search)
+
     self._searchbox.pack_start(slabel, False)
     self._search_result_label = gtk.Label()
     self._search_result_label.set_text("")
     self._search_result_label.show()
-    self._searchbox.pack_end(self._search_result_label, False)
     self._searchbox.pack_start(self._searchinput)
+    self._searchbox.pack_start(self._search_result_label, False)
+    self._searchbox.pack_start(self._search_next, False, False)
+    self._searchbox.pack_end(sclose, False, False)
 
     self.show()
     self.pack_start(self._titlebox, False)
@@ -102,6 +132,7 @@ class TerminatorTerm (gtk.VBox):
 
     if self.conf.titlebars:
       self._titlebox.show()
+      self._want_titlebar = True
     else:
       self._titlebox.hide()
 
@@ -288,7 +319,9 @@ text/plain
       txt = selection_data.data.strip()
       if txt[0:7] == "file://":
         txt = "'%s'" % txt[7:]
-      self._vte.feed_child(txt)
+      for term in self.terminator.term_list:
+        if term == self or (term._group != None and term._group == self._group):
+          term._vte.feed_child(txt)
       return
       
     widgetsrc = data.terminator.term_list[int(selection_data.data)]
@@ -308,7 +341,7 @@ text/plain
       return
     pos = self.get_location(widget, x, y)
     
-    data.terminator.remove(widgetsrc)
+    data.terminator.remove(widgetsrc, True)
     data.terminator.add(self, widgetsrc,pos)
     return
 
@@ -370,6 +403,8 @@ text/plain
       self.matches['addr_only'] = self._vte.match_add (lboundry + "(www|ftp)[" + hostchars + "]*\.[" + hostchars + ".]+(:[0-9]+)?(" + urlpath + ")?" + rboundry + "/?")
       self.matches['email'] = self._vte.match_add (lboundry + "(mailto:)?[a-zA-Z0-9][a-zA-Z0-9.+-]*@[a-zA-Z0-9][a-zA-Z0-9-]*\.[a-zA-Z0-9][a-zA-Z0-9-]+[.a-zA-Z0-9-]*" + rboundry)
       self.matches['nntp'] = self._vte.match_add (lboundry + '''news:[-A-Z\^_a-z{|}~!"#$%&'()*+,./0-9;:=?`]+@[-A-Za-z0-9.]+(:[0-9]+)?''' + rboundry)
+      # if the url looks like a Launchpad changelog closure entry LP: #92953 - make it a url to http://bugs.launchpad.net
+      self.matches['launchpad'] = self._vte.match_add ('\\bLP:? #?[0-9]+\\b')
 
   def _path_lookup(self, command):
     if os.path.isabs (command):
@@ -463,6 +498,7 @@ text/plain
     dbg ('SEGBUG: Forked command')
 
     self.on_vte_title_change(self._vte) # Force an initial update of our titles
+    self._title.show()
 
     if self._pid == -1:
       err (_('Unable to start shell: ') + shell)
@@ -472,7 +508,12 @@ text/plain
     """ Return the current working directory of the subprocess.
         This function requires OS specific behaviours
     """
-    cwd = self.terminator.pid_get_cwd (self._pid)
+    try:
+      cwd = self.terminator.pid_get_cwd (self._pid)
+    except OSError:
+      err ('get_cwd: unable to get cwd of %d' % self._pid)
+      cwd = '~'
+      pass
     dbg ('get_cwd found: %s'%cwd)
     return (cwd)
 
@@ -537,13 +578,18 @@ text/plain
     # Set our background image, transparency and type
     # Many thanks to the authors of gnome-terminal, on which this code is based.
     background_type = self.conf.background_type
+    dbg ('H9TRANS: Configuring background type as: %s' % background_type)
 
     # set background image settings
     if background_type == "image":
+      dbg ('H9TRANS: Setting background image to: %s' % self.conf.background_image)
       self._vte.set_background_image_file (self.conf.background_image)
+      dbg ('H9TRANS: Setting background image scroll to: %s' % self.conf.scroll_background)
       self._vte.set_scroll_background (self.conf.scroll_background)
     else:
+      dbg ('H9TRANS: Unsetting background image')
       self._vte.set_background_image_file('')
+      dbg ('H9TRANS: Unsetting background image scrolling')
       self._vte.set_scroll_background(False)
 
     # set transparency for the background (image)
@@ -552,15 +598,21 @@ text/plain
       self._vte.set_background_tint_color (bg_color)
       self._vte.set_background_saturation(1 - (self.conf.background_darkness))
       opacity = int(self.conf.background_darkness * 65535)
+      dbg ('H9TRANS: Set background tint color to: %s' % bg_color)
+      dbg ('H9TRANS: Set background saturation to: %s' % (1 - (self.conf.background_darkness)))
     else:
+      dbg ('H9TRANS: Set background saturation to: 1')
       self._vte.set_background_saturation(1)
       
     if self._composited_support:
+      dbg ('H9TRANS: Set opacity to: %s' % opacity)
       self._vte.set_opacity(opacity)
 
     if self._composited_support and not self._vte.is_composited():
+      dbg ('H9TRANS: Set background transparency to: %s' % (background_type == "transparent"))
       self._vte.set_background_transparent (background_type == "transparent")
     else:
+      dbg ('H9TRANS: Set background transparency to: False, hard')
       self._vte.set_background_transparent (False)
 
     # Set our cursor blinkiness
@@ -612,6 +664,9 @@ text/plain
         if url:
           if (url[0][0:7] != "mailto:") & (url[1] == self.matches['email']):
             address = "mailto:" + url[0]
+          elif url[1] == self.matches['launchpad']:
+            # the only part of 'launchpad' we need are the actual numbers for the bug
+            address = "https://bugs.launchpad.net/bugs/%s" % re.sub(r'[^0-9]+', '', url[0])
           else:
             address = url[0]
           self.openurl ( address )
@@ -620,6 +675,9 @@ text/plain
     # Left mouse button should transfer focus to this vte widget
     # we also need to give focus on the widget where the paste occured
     if event.button in (1 ,2):
+      if event.button == 2 and self._group:
+        self.paste_clipboard (True)
+        return True
       self._vte.grab_focus ()
       return False
 
@@ -637,6 +695,7 @@ text/plain
     self.toggle_widget_visibility (self._scrollbar)
 
   def do_title_toggle (self):
+    self._want_titlebar = not self._titlebox.get_property ('visible')
     self.toggle_widget_visibility (self._titlebox)
 
   def toggle_widget_visibility (self, widget):
@@ -648,21 +707,37 @@ text/plain
     else:
       widget.show ()
 
-  def paste_clipboard(self):
-    self._vte.paste_clipboard()
+  def paste_clipboard(self, primary = False):
+    if self._group:
+      for term in self.terminator.term_list:
+        if term._group == self._group:
+          if primary:
+            term._vte.paste_primary ()
+          else:
+            term._vte.paste_clipboard ()
+    else:
+      if primary:
+        self._vte.paste_primary ()
+      else:
+        self._vte.paste_clipboard ()
     self._vte.grab_focus()
 
   #keybindings for the individual splited terminals (affects only the
   #the selected terminal)
   UnhandledKeybindings = ('close_window', 'full_screen')
   def on_vte_key_press (self, term, event):
+    if not event:
+      dbg ('on_vte_key_press: Called on %s with no event' % term)
+      return False
     mapping = self.terminator.keybindings.lookup(event)
 
     if mapping and mapping not in self.UnhandledKeybindings:
-      dbg("on_vte_key_press: lookup found %s" % repr(mapping))
+      dbg("on_vte_key_press: lookup found %r" % mapping)
       getattr(self, "key_" + mapping)()
       return True
 
+    if self._group and self._vte.is_focus ():
+      self.terminator.group_emit (self, self._group, 'key-press-event', event)
     return False
 
   # Key events
@@ -700,6 +775,18 @@ text/plain
   def key_go_prev(self):
     self.terminator.go_prev (self)
 
+  def key_go_up(self):
+    self.terminator.go_up (self)
+
+  def key_go_down(self):
+    self.terminator.go_down (self)
+
+  def key_go_left(self):
+    self.terminator.go_left (self)
+
+  def key_go_right(self):
+    self.terminator.go_right (self)
+
   def key_split_horiz(self):
     self.terminator.splitaxis (self, False)
 
@@ -741,6 +828,12 @@ text/plain
 
   def key_prev_tab(self):
     self.terminator.previous_tab (self)
+
+  def key_reset(self):
+    self._vte.reset (True, False)
+
+  def key_reset_clear(self):
+    self._vte.reset (True, True)
   # End key events
 
   def zoom_orig (self):
@@ -769,7 +862,7 @@ text/plain
     if key == 'Escape':
       self.end_search()
 
-  def end_search(self):
+  def end_search(self, widget = None):
     self._search_row = 0
     self._search_string = None
     self._search_result_label.set_text("")
@@ -779,7 +872,7 @@ text/plain
 
   def do_search(self, widget):
     string = widget.get_text()
-    dbg("do_search: Looking for %s" % repr(string))
+    dbg("do_search: Looking for %r" % string)
     if string == '':
       return
 
@@ -794,21 +887,23 @@ text/plain
   def _search_character(self, widget, col, row, junk):
     return True
 
-  def next_search(self):
+  def next_search(self, widget=None):
     startrow,endrow = self._get_vte_buffer_range()
     while True:
       if self._search_row == endrow:
         self._search_row = startrow
         self._search_result_label.set_text("Finished Search")
+        self._search_next.hide()
         return
       buffer = self._vte.get_text_range(self._search_row, 0, self._search_row, -1, self._search_character)
 
-      # dbg("Row %d buffer: %s" % (self._search_row, repr(buffer)))
+      # dbg("Row %d buffer: %r" % (self._search_row, buffer))
       index = buffer.find(self._search_string)
       if index != -1:
         self._search_result_label.set_text("Found at row %d" % self._search_row)
         self._scrollbar.set_value(self._search_row)
         self._search_row += 1
+        self._search_next.show()
         return
       self._search_row += 1
 
@@ -817,6 +912,29 @@ text/plain
     startrow = max(0, endrow - self.conf.scrollback_lines)
     return(startrow, endrow)
 
+  def get_geometry (self):
+    '''Returns Gdk.Window.get_position(), pixel-based cursor position,
+       and Gdk.Window.get_geometry()'''
+    reply = dict()
+    x, y = self._vte.window.get_origin ()
+    reply.setdefault('origin_x',x)
+    reply.setdefault('origin_y',y)
+
+    column, row = self._vte.get_cursor_position ()
+    cursor_x = column * self._vte.get_char_width ()
+    cursor_y = row    * self._vte.get_char_height ()
+    reply.setdefault('cursor_x', cursor_x)
+    reply.setdefault('cursor_y', cursor_y)
+
+    geometry = self._vte.window.get_geometry()
+    reply.setdefault('offset_x', geometry[0])
+    reply.setdefault('offset_y', geometry[1])
+    reply.setdefault('span_x', geometry[2])
+    reply.setdefault('span_y', geometry[3])
+    reply.setdefault('depth', geometry[4])
+
+    return reply
+
   def create_popup_menu (self, widget, event = None):
     menu = gtk.Menu ()
     url = None
@@ -839,6 +957,9 @@ text/plain
           else:
               # Assume http
               address = "http://" + url[0]
+        elif url[1] == self.matches['launchpad']:
+          # the only part of 'launchpad' we need are the actual numbers for the bug 
+          address = "https://bugs.launchpad.net/bugs/%s" % re.sub(r'[^0-9]+', '', url[0])
         else:
           address = url[0]
         nameopen = _("_Open Link")
@@ -887,6 +1008,8 @@ text/plain
     item = gtk.CheckMenuItem (_("Show _titlebar"))
     item.set_active (self._titlebox.get_property ('visible'))
     item.connect ("toggled", lambda menu_item: self.do_title_toggle ())
+    if self._group:
+      item.set_sensitive (False)
     menu.append (item)
 
     self._do_encoding_items (menu)
@@ -954,6 +1077,24 @@ text/plain
       item = gtk.MenuItem ()
       menu.append (item)
 
+    item = gtk.MenuItem (_("Ed_it profile"))
+    item.connect ("activate", lambda menu_item: self.terminator.edit_profile (self))
+    menu.append (item)
+
+    item = gtk.MenuItem ()
+    menu.append (item)
+
+    item = gtk.MenuItem (_("_Group"))
+    menu.append (item)
+    submenu = gtk.Menu ()
+    item.set_submenu (submenu)
+    self.populate_grouping_menu (submenu)
+    if len (self.terminator.term_list) == 1:
+      item.set_sensitive (False)
+
+    item = gtk.MenuItem ()
+    menu.append (item)
+
     item = gtk.ImageMenuItem (gtk.STOCK_CLOSE)
     item.connect ("activate", lambda menu_item: self.terminator.closeterm (self))
     menu.append (item)
@@ -963,6 +1104,111 @@ text/plain
 
     return True
 
+  def populate_grouping_menu (self, widget):
+    groupitem = None
+
+    if len (self.terminator.groupings) > 0:
+      groupitem = gtk.RadioMenuItem (groupitem, _("None"))
+      groupitem.set_active (self._group == None)
+      groupitem.connect ("activate", self.set_group, None)
+      widget.append (groupitem)
+
+      for group in self.terminator.groupings:
+        item = gtk.RadioMenuItem (groupitem, group)
+        item.set_active (self._group == group)
+        item.connect ("toggled", self.set_group, group)
+        widget.append (item)
+        groupitem = item
+
+      item = gtk.MenuItem ()
+      widget.append (item)
+    
+    item = gtk.MenuItem (_("_New group"))
+    item.connect ("activate", self.create_group)
+    widget.append (item)
+
+    item = gtk.MenuItem ()
+    widget.append (item)
+
+    item = gtk.MenuItem (_("_Group all"))
+    item.connect ("activate", self.group_all)
+    widget.append (item)
+
+    if len (self.terminator.groupings) > 0:
+      item = gtk.MenuItem (_("_Ungroup all"))
+      item.connect ("activate", self.ungroup_all)
+      widget.append (item)
+
+  def create_group (self, item):
+    win = gtk.Window ()
+    vbox = gtk.VBox ()
+    hbox = gtk.HBox ()
+    entrybox = gtk.HBox ()
+    win.add (vbox)
+    label = gtk.Label (_("Group name:"))
+    entry = gtk.Entry ()
+    okbut = gtk.Button (stock=gtk.STOCK_OK)
+    canbut = gtk.Button (stock=gtk.STOCK_CANCEL)
+
+    entrybox.pack_start (label, False, True)
+    entrybox.pack_start (entry, True, True)
+    hbox.pack_end (okbut, False, False)
+    hbox.pack_end (canbut, False, False)
+    vbox.pack_start (entrybox, False, True)
+    vbox.pack_start (hbox, False, True)
+
+    canbut.connect ("clicked", lambda kill: win.destroy())
+    okbut.connect ("clicked", self.do_create_group, win, entry)
+    entry.connect ("activate", self.do_create_group, win, entry)
+
+    win.show_all ()
+
+  def do_create_group (self, widget, window, entry):
+    name = entry.get_text ()
+    self.terminator.groupings.append (name)
+    self.set_group (None, name)
+
+    window.destroy ()
+
+  def set_group (self, item, data):
+    if self._group == data:
+      # No action needed
+      return
+      
+    if data:
+      self._titlegroup.set_text (data)
+      self._titlegroup.show()
+      self._titlesep.show ()
+    else:
+      self._titlegroup.hide()
+      self._titlesep.hide ()
+
+    if not self._group:
+      # We were not previously in a group
+      self._titlebox.show ()
+      self._group = data
+    else:
+      # We were previously in a group
+      self._group = data
+      if data == None:
+        # We have been removed from a group
+        if not self.conf.titlebars and not self._want_titlebar:
+          self._titlebox.hide ()
+      self.terminator.group_hoover ()
+
+  def group_all (self, widget):
+    allname = _("All")
+    if not allname in self.terminator.groupings:
+      self.terminator.groupings.append (allname)
+    for term in self.terminator.term_list:
+      term.set_group (None, allname)
+    self.terminator.group_hoover ()
+
+  def ungroup_all (self, widget):
+    for term in self.terminator.term_list:
+      term.set_group (None, None)
+    self.terminator.group_hoover ()
+
   def on_encoding_change (self, widget, encoding):
     current = self._vte.get_encoding ()
     if current != encoding:
@@ -1028,12 +1274,17 @@ text/plain
 
   def on_vte_title_change(self, vte):
     title = self.get_window_title(vte)
+    if title == self._oldtitle:
+      # Title hasn't changed, don't bother doing anything
+      return
+    self._oldtitle = title
+
     if self.conf.titletips:
       vte.set_property ("has-tooltip", True)
       vte.set_property ("tooltip-text", title)
     #set the title anyhow, titlebars setting only show/hide the label
     self._title.set_text(title)
-    self.terminator.set_window_title("%s - %s" % (title, APP_NAME.capitalize()))
+    self.terminator.set_window_title("%s - %s" % (re.sub(' - %s' % APP_NAME.capitalize(), '', title), APP_NAME.capitalize()))
     notebookpage = self.terminator.get_first_notebook_page(vte)
     while notebookpage != None:
       if notebookpage[0].get_tab_label(notebookpage[1]):
@@ -1044,12 +1295,10 @@ text/plain
 
   def on_vte_focus_in(self, vte, event):
     self._titlebox.modify_bg(gtk.STATE_NORMAL,self.terminator.window.get_style().bg[gtk.STATE_SELECTED])
-    self._title.modify_fg(gtk.STATE_NORMAL, self.terminator.window.get_style().fg[gtk.STATE_SELECTED])
     return
 
   def on_vte_focus_out(self, vte, event):
     self._titlebox.modify_bg(gtk.STATE_NORMAL, self.terminator.window.get_style().bg[gtk.STATE_NORMAL])
-    self._title.modify_fg(gtk.STATE_NORMAL, self.terminator.window.get_style().fg[gtk.STATE_NORMAL])
     return
 
   def on_vte_focus(self, vte):
@@ -1063,6 +1312,3 @@ text/plain
         notebookpage[0].set_tab_label(notebookpage[1], label)
       notebookpage = self.terminator.get_first_notebook_page(notebookpage[0])
 
-  def destroy(self):
-    self._vte.destroy()
- 
